 

import greenfoot.*;
import java.awt.Color;
import java.lang.Math;

public class PixelFactory
{
	public static final Color[] defaltColors3 = {Color.lightGray, Color.GRAY, Color.darkGray};
	public static final Color[] defaltColors5 = {Color.WHITE, Color.lightGray, Color.GRAY, Color.darkGray, Color.BLACK};
	
	public static void scale(GreenfootImage GI, int s){
		GI.scale(GI.getWidth() * s, GI.getHeight() * s);
	}
	
	public static GreenfootImage extend(GreenfootImage GI ,int d){
		if(d < 0)
			d--;
		GreenfootImage buffer = new GreenfootImage(GI.getWidth() + 2 * d, GI .getHeight() + 2 * d);
		buffer.drawImage(GI, d, d);
		
		return buffer;
	}
	
	public static GreenfootImage extend(GreenfootImage GI, float r){
		return extend(GI, Math.round(Math.max(GI.getWidth(), GI.getHeight()) * r));
	}
	
	public static GreenfootImage stroke(GreenfootImage GI, Color color,float sW, boolean solid){
		int strokeWidth = (int)sW;
		GreenfootImage buffer = new GreenfootImage(GI.getWidth() + 2 * strokeWidth, GI.getHeight() + 2 * strokeWidth);
		buffer.drawImage(GI, strokeWidth, strokeWidth);
		
		for(int i = 0; i < buffer.getWidth(); i++)
		for(int j = 0; j < buffer.getHeight(); j++){
			if(buffer.getColorAt(i, j).getAlpha() == 0)
			for(int dx = -strokeWidth; dx <= strokeWidth; dx++)
			for(int dy = -strokeWidth; dy <= strokeWidth; dy++){
				int x = i + dx - strokeWidth;
				int y = j + dy - strokeWidth;
				double distance = Math.sqrt(dx * dx + dy * dy);
				if(distance <= sW
				&& x >= 0 && x < GI.getWidth()
				&& y >= 0 && y < GI.getHeight())
				if(GI.getColorAt(x, y).getAlpha() != 0){
					int transparency = (int)(255f / sW * (strokeWidth + 1 - distance));
					if(solid)
					if(distance <= strokeWidth)
						transparency = 255;
					transparency = Math.min(transparency, 255);
					if(transparency > buffer.getColorAt(i, j).getAlpha())
					buffer.setColorAt(i, j, new Color(color.getRed(), color.getGreen(), color.getBlue(), transparency));
				}
			}
		}
		
		return buffer;
	}
	public static GreenfootImage stroke(GreenfootImage GI, Color color, float sW){
		return stroke(GI, color, sW, false);
	}
	
	public static void replace(GreenfootImage GI,Color[] aim, Color[] bullet){
		for(int i = 0; i < GI.getWidth(); i++)
		for(int j = 0; j < GI.getHeight(); j++)
		for(int id = 0; id < aim.length; id++)
			if(GI.getColorAt(i, j).equals(aim[id])){
				GI.setColorAt(i, j, bullet[id]);
				//return;
			}
	}
	
	
	
	
	
	
	public static GreenfootImage ColorBar(int width){
		GreenfootImage GI = new GreenfootImage(361, 1);
		//GI.fill();
		for(int i = 0; i < 361; i++){
			GI.setColorAt(i, 0, new Color(Color.HSBtoRGB(i, 1, 1)));
		}
		GI.scale(width, 9);
		return GI;
	}
	
	public static GreenfootImage Ball(int radius){
		int size = radius * 2 + 1;
		GreenfootImage GI = new GreenfootImage(size, size);
		//GI.setColor(color);
		//GI.fill();
		GI.setColor(defaltColors3[2]);
		GI.fillOval(0, 0, size, size);
		
		float scaleRate = 0.72f;
		float stepRate = (float)((1 - scaleRate) * (Math.sqrt(2) - 1) * Math.sqrt(2) / 4 + 0.12 * Math.sqrt(2) / 4);
		int size0 = Math.round(size * scaleRate);
		int d = Math.round(size * stepRate);
		GreenfootImage buffer = new GreenfootImage(size0, size0);
		buffer.setColor(defaltColors3[1]);
		buffer.fillOval(0, 0, size0, size0);
		GI.drawImage(buffer, d, d);
		
		size0 = Math.round(size0 * scaleRate * scaleRate);
		d += (int)Math.round(size * stepRate);
		buffer = new GreenfootImage(size0, size0);
		buffer.setColor(defaltColors3[0]);
		buffer.fillOval(0, 0, size0, size0);
		GI.drawImage(buffer, d, d);
		
		return GI;
	}
	
	public static GreenfootImage Brick(int size, int stroke){
		GreenfootImage GI = new GreenfootImage(size, size);
		//GI.fill();
		GI.setColor(defaltColors3[1]);
		GI.fillRect(stroke / 2, stroke / 2, size - stroke, size - stroke);
		for(int i = stroke / 2; i < size - stroke / 2; i++)
		for(int j = stroke / 2; j < size - stroke / 2; j++){
			if(Greenfoot.getRandomNumber(3) > 1){
				int c = 0;
				if(Greenfoot.getRandomNumber(size - stroke / 2) + Greenfoot.getRandomNumber(size - stroke / 2) < i + j)
					c += 2;
				GI.setColorAt(i, j, defaltColors3[c]);
			}
		}
		
		GI.setColor(defaltColors3[0]);
		GI.fillRect(0, stroke, stroke, size - stroke * 2);
		GI.fillRect(stroke, 0, size - stroke * 2, stroke);
		
		GI.setColor(defaltColors3[2]);
		GI.fillRect(size - stroke, stroke, size, size - stroke * 2);
		GI.fillRect(stroke, size - stroke, size - stroke * 2, size);
		
		return GI;
	}
	
	public static GreenfootImage Eye(int radius, float theta){
		int size = radius * 2 + 1;
		float Width = size * 0.9f;
		float Height = size * 0.5f;
		float scaleRate = (float)Math.cos(Math.toRadians((double)theta));
		float scaledWidth = Width * scaleRate;
		float centerHeight = Height * scaleRate;
		float leftHeight = centerHeight + scaledWidth / 2 * (1 - scaleRate);
		float rightHeight = centerHeight - scaledWidth / 2 * (1 -scaleRate);
		
		GreenfootImage GI =new GreenfootImage(Math.round(scaledWidth), Math.round(Math.max(leftHeight, rightHeight)));
		GI.setColor(Color.WHITE);
		//GI.fill();
		int[] x = {0, GI.getWidth(), GI.getWidth(), 0};
		float centerY = Math.round(GI.getHeight() / 2f);
		int[] y = {Math.round(centerY - leftHeight / 2), Math.round(centerY - rightHeight / 2), Math.round(centerY + rightHeight / 2), Math.round(centerY + leftHeight / 2)};
		GI.fillPolygon(x, y, 4);
		
		
		
		return GI;
	}
	
	public static GreenfootImage Cloud(int size){
		float width = 1.6f;
		GreenfootImage GI = new GreenfootImage(Math.round(size * width), size);
		//GI.fill();
		
		GI.setColor(defaltColors3[2]);
		int i0 = Math.round(size * 5f / 8f);
		int i1 = Math.round(size * 15f / 16f);
		int x0 = Math.round(size * 9f / 8f);
		for(int i = i0; i < i1; i++){
			float rate = (float)(i - i0) / (float)(i1 - i0);
			int x1 = Math.round(x0 + (size * width - x0) * rate * rate * rate * rate);
			//x1 = Math.round(1.4f * size);
			GI.drawLine(Math.round(size / 4f), i, x1,i);
		}
		
		GI.drawImage(PixelFactory.Ball(Math.round(size / 4f)), 0, Math.round(size / 2f - 1));
		GI.drawImage(PixelFactory.Ball(Math.round(size * 3f / 8f)), Math.round(size / 8f), 0);
		GI.drawImage(PixelFactory.Ball(Math.round(size * 5f / 16f)), Math.round(size / 2f), Math.round(size * 5f / 16f));
		
		return GI;
	}
}
