package greenfoot;

import android.content.pm.ActivityInfo;
import android.graphics.PointF;
import android.view.MotionEvent;
import android.view.View;
import java.util.ArrayList;
import java.util.List;
import java.util.Random;

/**
*@author Upz
*a class to orgnize things together
*/
public final class Greenfoot{
	static boolean debug = false;
	
	public static GreenfootApp activity;
	static GameView view;
	static World currentWorld;
	static GameThread thread;
	
	static float worldFrameX;
	static float worldFrameY;
	static float worldFrameW;
	static float worldFrameH;
	static float scaleRate;
	
	public static final int mouseMod = 0;
	public static final int keyBoardMod = 1;
	static int inputMod = 0;
	static TouchManager touchManager;
	static List<PointF> touchPoints =  new ArrayList<PointF>();
	
	static 	boolean Running;
	
	static final Random random = new Random();
	
	static List<String> errLines = new ArrayList<String>();
	
	Greenfoot(){
	}
	
	static void init(GreenfootApp ga){
		Running = false;
		activity = ga;
		UserInfo.init();
		
		view = new GameView(activity);
		view.init();
		activity.setContentView(view);
	}
	
	public static void setWorld(World world){
		currentWorld = world;
		currentWorld.init();
		
		if(getWorldWidthPx() > getWorldHeightPx())
			activity.setRequestedOrientation(ActivityInfo.SCREEN_ORIENTATION_LANDSCAPE);
	
		view.currentWorld = world;
		if(thread != null)
		thread.currentWorld = world;
	}
	
	static void viewCreated(){
		creatThread();

		placeWorldFrame();
		setScaleRate();
		touchManager = new TouchManager();
	}
	
	static void creatThread(){
		thread = new GameThread();
		thread.init();
		thread.start();
	}
	
	public static void start(){
		Running = true;
	}
	
	public static void stop(){
		if(thread != null)
		thread.isStopped = true;
		Running = false;
	}
	
	public static void setSpeed(int speed){
		thread.speed = speed;
	}
	
	static void destroyThread(){
		if(thread != null)
			thread.setAlive(false);
	}
	
	static void placeWorldFrame(){
		worldFrameX = getViewWidth() / 2.0f;
		worldFrameY = getViewHeight() / 2.0f;
		worldFrameW = getViewWidth();
		worldFrameH = getViewHeight();
	}
	
	static void placeWorldFrameAt(float x, float y){
		worldFrameX = x;
		worldFrameY = y;
		worldFrameW = Math.min(worldFrameX, getViewWidth() - worldFrameX) * 2f;
		worldFrameH = Math.min(worldFrameY, getViewHeight() - worldFrameY) * 2f;
	}
	
	static void placeWorldFrameIn(float left, float top, float width, float height){
		worldFrameX = left + width / 2;
		worldFrameY = top + height / 2;
		worldFrameW = width;
		worldFrameH = height;
	}
	
	static void setScaleRate(){
		float sx = worldFrameW / getWorldWidthPx();
		float sy = worldFrameH / getWorldHeightPx();
		scaleRate = Math.min(sx, sy);
	}
	
	void setScaleRate(float sr){
		scaleRate = sr;
	}
	
	static void loadRecord(){
		touchManager.currentKeyInfo = touchManager.bufferedKeyInfo;
		//touchManager.bufferedKeyInfo = new KeyInfo();
		
		float x = touchManager.currentMouseInfo.x;
		float y = touchManager.currentMouseInfo.y;
		
		if(touchManager.bufferedMouseInfo.lastEvent != null){
			PointF sp = new PointF(touchManager.bufferedMouseInfo.lastEvent.getX(), touchManager.bufferedMouseInfo.lastEvent.getY());
			PointF wp = ScreenPointToWorld(sp);
			if(isInWorld(wp)){
				x = wp.x;
				y = wp.y;
			}
			
			touchPoints.clear();
			for(int i = 0; i < touchManager.bufferedMouseInfo.lastEvent.getPointerCount(); i++){
				touchPoints.add(new PointF(touchManager.bufferedMouseInfo.lastEvent.getX(i),
				                           touchManager.bufferedMouseInfo.lastEvent.getY(i)));
			}
		}
		
		
		touchManager.currentMouseInfo = touchManager.bufferedMouseInfo;
		touchManager.currentMouseInfo.x = x;
		touchManager.currentMouseInfo.y = y;
		touchManager.bufferedMouseInfo = new MouseInfo();
	}
	
	static float getWorldFrameLeft(){
		return worldFrameX - worldFrameW / 2.0f;
	}
	
	static float getWorldFrameTop(){
		return worldFrameY - worldFrameH / 2.0f;
	}
	
	static float getWorldFrameRight(){
		return worldFrameX + worldFrameW / 2.0f;
	}
	
	static float getWorldFrameBottom(){
		return worldFrameY + worldFrameH / 2.0f;
	}
	
	
	
	
	
	
	
	public static boolean isWorldLandScape(){
		return getWorldWidthPx() >= getWorldHeightPx();
	}
	
	public static PointF ScreenPointToWorld(PointF screenPoint){
		float offsetX = getWorldFrameLeft() + (worldFrameW - getWorldWidthPx() * scaleRate) / 2;
		float offsetY = getWorldFrameTop() + (worldFrameH - getWorldHeightPx() * scaleRate) / 2;
		float tpX = screenPoint.x - offsetX;
		float tpY = screenPoint.y - offsetY;
		tpX /= scaleRate * currentWorld.cellSize;
		tpY /= scaleRate * currentWorld.cellSize;
		return new PointF(tpX, tpY);
	}
	
	public static boolean isInWorld(PointF wp){
		return (0 <= wp.x && wp.x <= getWorldWidthPx() / currentWorld.cellSize
			&& 0 <= wp.x && wp.y <= getWorldHeightPx() / currentWorld.cellSize);
	}
	
	
	public static Actor getTopActorAt(float px, float py){
		List<Actor> actors = currentWorld.paintList;
		for(int i = actors.size(); i > 0; i--){
			Actor actor = actors.get(i - 1);
			if(actor.isContain(px, py))
				return actor;
		}
		return null;
	}
	
	public static MouseInfo getMouseInfo(){
		if(touchManager.currentMouseInfo.isOutOfWorld)
			return null;
		else
		    return touchManager.currentMouseInfo;
	}
	
	static void getLastRelatedActor(){
		if(!touchManager.currentMouseInfo.isDragged){
			touchManager.relatedActor = null;
			touchManager.relatedActor = getTopActorAt(touchManager.currentMouseInfo.x, touchManager.currentMouseInfo.y);
		}
	}
	
	public static boolean isKeyDown(String keyName){
		return touchManager.currentKeyInfo.isKeyDown(keyName);
	}
	
	public static String getKey(){
		String lk = touchManager.currentKeyInfo.lastKey;
		touchManager.currentKeyInfo.lastKey = null;
		touchManager.bufferedKeyInfo.lastKey = null;
		return lk;
	}
	
	public static float getTouchX(){
		return touchManager.currentMouseInfo.x;
	}
	
	public static float getTouchY(){
		return touchManager.currentMouseInfo.y;
	}
	
	public static boolean mouseClicked(Object object){
		return (touchManager.currentMouseInfo.isReleased
		    && (object == null || object == touchManager.relatedActor));
	}
	
	public static boolean mousePressed(Object object){
		return (touchManager.currentMouseInfo.isTouching
		    && (object == null || object == touchManager.relatedActor));
	}
	
	public static boolean mouseDragged(Object object){
		return (touchManager.currentMouseInfo.isDragged
		    && (object == null || object == touchManager.relatedActor));
	}
	
	static boolean isLongPressed(){
		return touchManager.currentMouseInfo.isLongPressed;
	}
	
	public static void setInputMod(int im){
		inputMod = im;
	}
	
	
	
	
	static int getViewWidth(){
		return view.getWidth();
	}
	
	static int getViewHeight(){
		return view.getHeight();
	}
	
	static float getWorldWidthPx(){
		return currentWorld.WidthPx;
	}
	
	static float getWorldHeightPx(){
		return currentWorld.HeightPx;
	}
	
	static float getWorldCellSize(){
		return currentWorld.cellSize;
	}
	
	
	
	
	public static int getRandomNumber(int limit){
		return random.nextInt(limit);
	}
	
	static void showInfo(String info){
		view.addExtraLines(info);
	}
	
	static void addErrLines(Exception e){
		//errLines.add("");
		errLines.add(e.toString());
		StackTraceElement ee[] = e.getStackTrace();
		if(ee.length > 0)
			for(int i = 0; i < ee.length; i++){
				String errorInfo = ee[i].getFileName() + "  ( " + ee[i].getMethodName() + " )";//___  " + ee[i].getLineNumber();
				errLines.add(errorInfo);
				System.out.println(ee[i]);
			}
	}
}
	
