package greenfoot;

import android.graphics.PointF;
import android.view.GestureDetector;
import android.view.View;
import android.view.View.OnTouchListener;
import android.view.MotionEvent;
import java.util.List;
import java.util.ArrayList;

public class TouchManager implements OnTouchListener
{
	GestureDetector gd;
	static MouseInfo bufferedMouseInfo;
	static MouseInfo currentMouseInfo;
	static MouseInfo lastMouseInfo;
	static boolean isTouching;
	static boolean isDragging;
	static Actor relatedActor;
	static Actor draggingActor;
	
	static float x0;
	static float x1;
	static float x2;
	static float x3;
	static float y0;
	static float y1;
	static float y2;
	static float y3;
	static KeyInfo bufferedKeyInfo;
	static KeyInfo currentKeyInfo;
	
	//static MotionEvent lastEvent;
	static List<PointF> pointers;
	
	public TouchManager(){
		pointers = new ArrayList<PointF>();
		
		float L = Greenfoot.getWorldFrameLeft();
		float W = Greenfoot.getWorldFrameRight() - L;
		float T = Greenfoot.getWorldFrameTop();
		float H = Greenfoot.getWorldFrameBottom() - T;
		x0 = L;
		x1 = L + 0.3f * W;
		x2 = L + 0.7f * W;
		x3 = Greenfoot.getWorldFrameRight();
		y0 = T;
		y1 = T + 0.3f * H;
		y2 = T + 0.7f * H;
		y3 = Greenfoot.getWorldFrameBottom();
		currentKeyInfo = new KeyInfo();
		bufferedKeyInfo = new KeyInfo();
		
		gd = new GestureDetector(Greenfoot.activity, new GameGestureListener());
		gd.setIsLongpressEnabled(true);
		gd.setOnDoubleTapListener(new GameGestureListener());
		
		currentMouseInfo = new MouseInfo();
		bufferedMouseInfo = new MouseInfo();
	}
	
	@Override
	public boolean onTouch(View v, MotionEvent event){
		updatePointers(event);
			
		if(Greenfoot.inputMod == Greenfoot.mouseMod)
			onMouseMod(event);
		else if(Greenfoot.inputMod == Greenfoot.keyBoardMod)
		    onKeyBoardMod(event);
		return true;
	}
	
	void updatePointers(MotionEvent event){
		pointers.clear();
		for(int i = 0; i < event.getPointerCount(); i++)
			pointers.add(new PointF(event.getX(i), event.getY(i)));
			
		if(event.getAction() == MotionEvent.ACTION_POINTER_1_UP)
			pointers.remove(0);
		if(event.getAction() == MotionEvent.ACTION_POINTER_2_UP)
			pointers.remove(1);
		if(event.getAction() == MotionEvent.ACTION_POINTER_3_UP)
			pointers.remove(2);
		if(event.getAction() == MotionEvent.ACTION_UP && event.getPointerCount() == 1)
			pointers.clear();
	}
	
	void onMouseMod(MotionEvent event){
		PointF tp = Greenfoot.ScreenPointToWorld(new PointF(event.getX(), event.getY()));
		if(Greenfoot.isInWorld(tp)){//now tp is ok to use
			bufferedMouseInfo.lastEvent = event;
			if((event.getAction() == MotionEvent.ACTION_DOWN)
			   || (event.getAction() == MotionEvent.ACTION_MOVE)){
				if(!currentMouseInfo.isTouching)
					bufferedMouseInfo.isEntered = true;
				bufferedMouseInfo.isTouching = true;
			}

			if((event.getAction() == MotionEvent.ACTION_UP)
			   || (event.getAction() == MotionEvent.ACTION_CANCEL)){
				if(currentMouseInfo.isTouching)// && !currentMouseInfo.isDragged)
					bufferedMouseInfo.isReleased = true;

				if(currentMouseInfo.isDragged = true)
					bufferedMouseInfo.isDragEnded = true;

				bufferedMouseInfo.isTouching = false;
				bufferedMouseInfo.isDragged = false;
			}
			gd.onTouchEvent(event);
			bufferedMouseInfo.isOutOfWorld = false;
		}
		else
			bufferedMouseInfo.isOutOfWorld = true;
	}
	
	void onKeyBoardMod(MotionEvent event){
		bufferedKeyInfo.left = false;
		bufferedKeyInfo.right = false;
		bufferedKeyInfo.up = false;
		bufferedKeyInfo.down = false;
		bufferedKeyInfo.space = false;
		for(int i = 0; i < pointers.size(); i++){
			float x = pointers.get(i).x;
			float y = pointers.get(i).y;
			if(x0 <= x && x <= x1)
				bufferedKeyInfo.left = true;
			if(x2 <= x && x <= x3)
				bufferedKeyInfo.right = true;
			if(y0 <= y && y <= y1)
				bufferedKeyInfo.up = true;
			if(y2 <= y && y <= y3)
				bufferedKeyInfo.down = true;
			if(x1 < x && x < x2 && y1 < y && y < y2)
				bufferedKeyInfo.space = true;
		}
		
		if(event.getAction() == MotionEvent.ACTION_DOWN){
			float x = event.getX(event.getActionIndex());
			float y = event.getY(event.getActionIndex());
			if(x0 <= x && x <= x1)
				bufferedKeyInfo.lastKey = "left";
			if(x2 <= x && x <= x3)
				bufferedKeyInfo.lastKey = "right";
			if(y0 <= y && y <= y1)
				bufferedKeyInfo.lastKey = "up";
			if(y2 <= y && y <= y3)
				bufferedKeyInfo.lastKey = "down";
			if(x1 < x && x < x2 && y1 < y && y < y2)
				bufferedKeyInfo.lastKey = "space";
		}
	}
}
